@@ -18,7 +18,7 @@ from unittest_mixins import change_dir
 import coverage
 from coverage.backward import unicode_class
 from coverage import env
-from coverage.files import flat_rootname
+from coverage.files import abs_file, flat_rootname
 import coverage.html
 from coverage.misc import CoverageException, NotPython, NoSource
 from coverage.report import get_analysis_to_report
@@ -635,6 +635,8 @@ def compare_html(expected, actual):
         # The temp dir the tests make.
         (filepath_to_regex(os.getcwd()), 'TEST_TMPDIR'),
         (filepath_to_regex(flat_rootname(unicode_class(os.getcwd()))), '_TEST_TMPDIR'),
+        (filepath_to_regex(abs_file(os.getcwd())), 'TEST_TMPDIR'),
+        (filepath_to_regex(flat_rootname(unicode_class(abs_file(os.getcwd())))), '_TEST_TMPDIR'),
         (r'/private/var/folders/[\w/]{35}/coverage_test/tests_test_html_\w+_\d{8}', 'TEST_TMPDIR'),
         (r'_private_var_folders_\w{35}_coverage_test_tests_test_html_\w+_\d{8}', '_TEST_TMPDIR'),
     ]
